import os
from tkinter import *
from tkinter import ttk
import sqlite3 as sql3
import pandas
import numpy

racine = Tk()
racine.attributes('-fullscreen', True)
racine.bind('<Escape>', lambda e: racine.destroy())

connection = sql3.connect('bdd_binnerto.db')
curseur = connection.cursor()


#################### Fonctions de début ####################
def make():
    connection.commit()


##########-CREATE-##########
def create_table_produits():  # FK!!
    curseur.execute(
        "CREATE TABLE IF NOT EXISTS PRODUITS(id_produit INTEGER PRIMARY KEY AUTOINCREMENT UNIQUE, nom_produit TEXT, prix NUMERIC, stock INTEGER)")
    make()


def create_table_clients():  # FK!!
    curseur.execute(
        "CREATE TABLE IF NOT EXISTS CLIENTS(id_client INTEGER PRIMARY KEY AUTOINCREMENT UNIQUE, nom TEXT, prenom TEXT, adresse_mail TEXT, adresse TEXT, date_naissance DATE, date_inscription DATE, vip bool)")
    make()


def create_table_commandes():  # FK!!
    curseur.execute(
        "CREATE TABLE IF NOT EXISTS COMMANDES(id_commande INTEGER PRIMARY KEY AUTOINCREMENT UNIQUE, id_client_commande INTEGER, id_produit INTEGER, date_commande DATE, date_livraison DATE, envoye bool, quantite INTEGER, prix_total FLOAT, FOREIGN KEY (id_client_commande) REFERENCES CLIENTS(id_client), FOREIGN KEY (id_produit) REFERENCES PRODUITS(id_produit))")
    make()


##########-ADD-##########
def add_produits(produits_ajout):
    curseur.executemany("INSERT INTO PRODUITS(nom_produit, prix, stock) VALUES (?,?,?)", produits_ajout)
    make()


def add_clients(clients_ajout):
    curseur.executemany(
        "INSERT INTO CLIENTS(nom, prenom, adresse_mail, adresse, date_naissance, date_inscription, vip) VALUES (?,?,?,?,?,?,?)",
        clients_ajout)
    make()


def add_commandes(commandes_ajout):
    curseur.executemany(
        "INSERT INTO COMMANDES(id_client_commande, id_produit, date_commande, date_livraison, envoye, quantite, prix_total) VALUES (?,?,?,?,?,?,?)",
        commandes_ajout)
    make()


##########-UPDATE & DELETE-##########
def update_commandes(id_commande, envoye: bool):
    curseur.execute("UPDATE COMMANDES SET envoye=? WHERE id_commande=?", (envoye, id_commande))
    make()


def update_produits(id_produit, stock: int):
    temp_stock = select_produit_specifique(id_produit)[1][3]
    curseur.execute("UPDATE PRODUITS SET stock=? WHERE id_produit=?", (str(temp_stock + stock), id_produit))
    make()


def update_clients(id_client, vip: bool):
    curseur.execute("UPDATE COMMANDES SET vip=? WHERE id_client=?", (vip, id_client))
    make()


def delete_commandes(id_commande):
    curseur.execute("DELETE FROM COMMANDES WHERE id_commande=?", (id_commande,))
    make()


def delete_produits(id_produit):
    curseur.execute("DELETE FROM PRODUITS WHERE id_produit=?", (id_produit,))
    make()


def delete_clients(id_client):
    curseur.execute("DELETE FROM CLIENTS WHERE id_client=?", (id_client,))
    make()


##########-READ-##########
def select_all_produits():
    curseur.execute("SELECT * FROM PRODUITS")
    return [("id_produit", "nom_produit", "prix", "stock")] + curseur.fetchall()


def select_all_clients():
    curseur.execute("SELECT * FROM CLIENTS")
    return [("id_client", "nom", "prenom", "adresse_mail", "adresse", "date_naissance", "date_inscription",
             "vip")] + curseur.fetchall()


def select_all_commandes():
    curseur.execute(
        "SELECT nom, prenom, nom_produit, date_commande, date_livraison, envoye, quantite FROM COMMANDES AS co INNER JOIN PRODUITS AS p ON co.id_produit = p.id_produit LEFT JOIN CLIENTS AS cl ON co.id_client_commande = cl.id_client")
    return [("nom", "prenom", "nom_produit", "date_commande", "date_livraison", "envoye",
             "quantite")] + curseur.fetchall()


def select_all_commandes_classique():
    curseur.execute("SELECT * FROM COMMANDES")
    return [("id_client_commande", "id_produit", "date_commande", "date_livraison", "envoye", "quantite", "prix_total")] + curseur.fetchall()


def select_produit_specifique(id_produit):
    curseur.execute("SELECT * FROM PRODUITS WHERE id_produit=?", (id_produit,))
    return [("id_produit", "nom_produit", "prix", "stock")] + curseur.fetchall()


def select_client_specifique(id_client):
    curseur.execute("SELECT * FROM CLIENTS WHERE id_client=?", (id_client,))
    return [("id_client", "nom", "prenom", "adresse_mail", "adresse", "date_naissance", "date_inscription",
             "vip")] + curseur.fetchall()


def select_commande_specifique(id_commande):
    curseur.execute(
        "SELECT nom, prenom, nom_produit, date_commande, date_livraison, envoye, quantite FROM COMMANDES AS co INNER JOIN PRODUITS AS p ON co.id_produit = p.id_produit LEFT JOIN CLIENTS AS cl ON co.id_client_commande = cl.id_client WHERE id_commande=?",
        (id_commande,))
    return [("nom", "prenom", "nom_produit", "date_commande", "date_livraison", "envoye",
             "quantite")] + curseur.fetchall()


def toutes_commandes_clients(id_client):
    curseur.execute(
        "SELECT nom, prenom, nom_produit, date_commande, date_livraison, envoye, quantite FROM COMMANDES AS co INNER JOIN PRODUITS AS p ON co.id_produit = p.id_produit LEFT JOIN CLIENTS AS cl ON co.id_client_commande = cl.id_client WHERE id_client_commande=?",
        (id_client,))
    return [("nom", "prenom", "nom_produit", "date_commande", "date_livraison", "envoye",
             "quantite")] + curseur.fetchall()


##########-ROUTINE CSV-##########
def open_csv(filename):
    if filename[-3:] == "csv":
        return pandas.read_csv(filename)
    else:
        return "Erreur, merci de réessayer"


def export_csv_table(csv_lu, table_destination):
    for i in range(len(csv_lu)):
        temp = list(csv_lu.loc[i])
        for j in range(len(temp)):
            if str(temp[j]) == "nan":
                temp[j] = None

            if type(temp[j]) == numpy.bool_:
                temp[j] = str(temp[j]).upper()

            if type(temp[j]) == numpy.int64:
                temp[j] = int(temp[j])

            if type(temp[j]) == numpy.float64 and str(temp[j] != "nan"):
                temp[j] = float(temp[j])
        if table_destination == "PRODUITS":
            add_produits([tuple(temp)])
        elif table_destination == "CLIENTS":
            add_clients([tuple(temp)])
        else:
            add_commandes([tuple(temp)])


def sauvegarde_csv():
    temp_produits = numpy.array(select_all_produits()[1:])
    temp_produits_df = pandas.DataFrame(temp_produits, index=[i for i in range(len(temp_produits))],
                                        columns=["id_produit", "nom_produit", "prix", "stock"],)
    temp_produits_df.to_csv("produits_s.csv")

    temp_clients = numpy.array(select_all_clients()[1:])
    temp_clients_df = pandas.DataFrame(temp_clients, index=[j for j in range(len(temp_clients))],
                                       columns=["id_client", "nom", "prenom", "adresse_mail", "adresse",
                                                "date_naissance", "date_inscription", "vip"], )
    temp_clients_df.to_csv("clients_s.csv")

    temp_commandes = numpy.array(select_all_commandes_classique()[1:])
    temp_commandes_df = pandas.DataFrame(temp_commandes, index=[k for k in range(len(temp_commandes))],
                                         columns=["id_commande", "id_client_commande", "id_produit", "date_commande",
                                                  "date_livraison", "envoye", "quantite", "prix_total"])
    temp_commandes_df.to_csv("commandes_s.csv")
    curseur.close()
    connection.close()
    os.remove("bdd_binnerto.db")
    racine.destroy()


#################### Code autre ####################
create_table_produits()
export_csv_table(open_csv("produits.csv"), "PRODUITS")

create_table_clients()
export_csv_table(open_csv("clients.csv"), "CLIENTS")

create_table_commandes()
export_csv_table(open_csv("commandes.csv"), "COMMANDES")


def bg_label():
    label.config(bg='#EEE3FF')


def action(recup):
    print(listeCombo.get())


def suppr(recup):
    del liste[listeCombo.get()]


def validate(test):
    return test.isdigit()


def action_add_stock_produit(event):
    if event.widget.get() == listeproduits[0]:
        update_produits(1, 1)
    elif event.widget.get() == listeproduits[1]:
        update_produits(2, 1)
    elif event.widget.get() == listeproduits[2]:
        update_produits(3, 1)
    elif event.widget.get() == listeproduits[3]:
        update_produits(4, 1)
    elif event.widget.get() == listeproduits[4]:
        update_produits(5, 1)
    elif event.widget.get() == listeproduits[5]:
        update_produits(6, 1)
    else:
        update_produits(7, 1)
    pop_up = Toplevel(racine)
    pop_up.title("Le stock a été mise a jour de 1.")
    label_temp = Label(pop_up, text="Le stock a été mise a jour de 1.", font=("Courier", 25))
    label_temp.pack()


labelChoix = Label(racine, text="Ajouter Stock :", bg='#EEE3FF')
labelChoix.place(x=205, y=490)
listeproduits = ["Parpaing externe", "Clé USHellB", "Clavier Pasteur STL", "Voiture à Hydrogène", "CPU Nucléaire Leif",
                 "Bouton d'allumage automatique", "Binnerto en acier", "Souris révolutionnaire"]
listeCombo = ttk.Combobox(racine, state="readonly", values=listeproduits)
listeCombo.bind("<<ComboboxSelected>>", action_add_stock_produit)
listeCombo.place(x=170, y=530)


def num_commande(event):
    n_fen = Toplevel(racine)
    t = Tableau(n_fen, select_commande_specifique(int(entree.get())))


com_liv = Label(racine, text="Afficher la commande d'un client (rentrez son ID de commande)", bg='#EEE3FF')
com_liv.place(x=590, y=490)
choix_com = racine.register(validate)
value = StringVar(racine)
entree = Entry(racine, textvariable=value, width=30, validate = 'key', validatecommand=(choix_com, '%S'))
entree.place(x=630, y=530)
entree.bind("<Return>", num_commande)


def ajout_commande(event):
    pop_up = Toplevel(racine)
    temp = event.widget.get()
    temp = str(temp)
    temp_liste = []
    temp_val_liste = ""
    for i in range(len(temp)):
        if i == len(temp)-1:
            temp_liste.append(temp_val_liste + temp[i])
        if temp[i] == ",":
            temp_liste.append(temp_val_liste)
            temp_val_liste = ""
        else:
            temp_val_liste += temp[i]
    print(len(temp_liste), temp_liste)
    if len(temp_liste) == 6:
            add_commandes([tuple(temp_liste + [None])])
            pop_up.title("La commande a bien été ajoutée")
            label_temp = Label(pop_up, text="Commande bien ajoutée", font=("Courier", 25))
            label_temp.pack()
    else:
            pop_up.title("La commande n'a pas pu être ajoutée, car votre ajout était trop court ou trop long")
            label_temp = Label(pop_up, text="Commande non ajoutée", font=("Courier", 25))
            label_temp.pack()
    

labelChoix = Label(racine, text="Ajouter la commande d'un client (rentrez l'ID client, l'ID produit,", bg='#EEE3FF')
labelChoix.place(x=900, y=120)

labelChoix2 = Label(racine, text="la date de commande, la date de livraison, si le produit a été livré,", bg='#EEE3FF')
labelChoix2.place(x=900, y=140)

labelChoix3 = Label(racine, text=" ainsi que la quantité de produits livrés, tous les éléments séparés par des virgules)", bg='#EEE3FF')
labelChoix3.place(x=900, y=160)

choix_suppr_com = racine.register(validate)
value_suppr = StringVar(racine)
entree_suppr = Entry(racine, width=30)
entree_suppr.place(x=940, y=180)
entree_suppr.bind("<Return>", ajout_commande)


def suppr_commande(event):
    delete_commandes(event.widget.get())
    pop_up = Toplevel(racine)
    pop_up.title("La commande a bien été supprimée")
    label_temp = Label(pop_up, text="Commande bien supprimée", font=("Courier", 25))
    label_temp.pack()

labelChoix = Label(racine, text="Supprimer la commande d'un client (rentrez son ID de commande)", bg='#EEE3FF')
labelChoix.place(x=990, y=490)
choix_suppr_com = racine.register(validate)
value_suppr = StringVar(racine)
entree_suppr = Entry(racine, textvariable=value_suppr, width=30, validate = 'key', validatecommand=(choix_suppr_com, '%S'))
entree_suppr.place(x=1040, y=530)
entree_suppr.bind("<Return>", suppr_commande)


def afficher_commandes():
    n_fen = Toplevel(racine)
    t = Tableau(n_fen, select_all_commandes())


button_afficher_commandes = Button(racine, text="Afficher toutes les commandes", width=24, height=7, bg='white', command=afficher_commandes)
button_afficher_commandes.place(x=160, y=120)


def afficher_produits():
    n_fen = Toplevel(racine)
    t = Tableau(n_fen, select_all_produits())


button_afficher_produits = Button(racine, text="Afficher tous les produits", width=24, height=7, bg='white', command=afficher_produits)
button_afficher_produits.place(x=160, y=250)


def afficher_clients():
    n_fen = Toplevel(racine)
    t = Tableau(n_fen, select_all_clients())


button_afficher_clients = Button(racine, text="Afficher touts les clients", width=24, height=7, bg='white', command=afficher_clients)
button_afficher_clients.place(x=440, y=120)


class Tableau:
    def __init__(self, racine, liste):
        for i in range(len(liste)):
            for j in range(len(liste[0])):
                self.tab = Entry(racine, font=20)
                self.tab.grid(row=i, column=j)
                self.tab.insert(END, liste[i][j])
                self.tab.configure(state='disabled')

def drop_data():
    curseur.execute("DELETE FROM 'COMMANDES'")
    curseur.execute("DELETE FROM 'CLIENTS'")
    curseur.execute("DELETE FROM 'PRODUITS'")


button_drop = Button(racine, text="Tout supprimer", width=24, height=7, bg='white', fg='red', command=drop_data)
button_drop.place(x=440, y=250)


def nouvelle_fenetre():
    n_fen = Toplevel(racine)
    t = Tableau(n_fen, liste)


bouton_terminer = Button(racine, text='Fermer et sauvegarder', bg='white', fg='green', command=sauvegarde_csv, font=("Courier", 17))
bouton_terminer.place(x=570,y=640)


racine['bg'] = '#EEE3FF'
racine.mainloop()
